#include "banking_lib.h"

Account& find_account(Ledger& ledger, acct_no_t acct_no)
{
    size_t low = 0, high = ledger.accounts.size();

    while (low < high) {
        size_t mid = (low + high) / 2;
        if (acct_no < ledger.accounts[mid].id) {
            high = mid;
        } else if (acct_no > ledger.accounts[mid].id) {
            low = mid + 1;
        } else {
            return ledger.accounts[mid];
        }
    }

    throw Bank_error("account not found");
}

const Account& find_account(const Ledger& ledger, acct_no_t acct_no)
{
    size_t low = 0, high = ledger.accounts.size();

    while (low < high) {
        size_t mid = (low + high) / 2;
        if (acct_no < ledger.accounts[mid].id) {
            high = mid;
        } else if (acct_no > ledger.accounts[mid].id) {
            low = mid + 1;
        } else {
            return ledger.accounts[mid];
        }
    }

    throw Bank_error("account not found");
}

void execute_transaction(Account& account, const Transaction& transaction)
{
    if (account.id != transaction.acct_id)
        throw Bank_error("transaction id does not match");

    switch (transaction.type) {
        case Transaction_type::credit:
            account.balance += transaction.amount;
            break;
        case Transaction_type::debit:
            account.balance -= transaction.amount;
            break;
    }
}

void clear_transactions(Ledger& ledger)
{
    for (const Transaction& transaction : ledger.transactions) {
        Account& account = find_account(ledger, transaction.acct_id);
        execute_transaction(account, transaction);
    }

    ledger.transactions.clear();
}
