#include "banking_lib.h"
#include <UnitTest++/UnitTest++.h>

Ledger make_test_ledger()
{
    Account acct1{1, 100,    "Jesse"};
    Account acct2{2, 10'000, "Shu-Hung"};
    Account acct3{4, 15'000, "Majed"};

    Transaction trans1{1, 10,   Transaction_type::credit, 1};
    Transaction trans2{1, 30,   Transaction_type::credit, 3};
    Transaction trans3{4, 3000, Transaction_type::debit,  5};

    std::vector<Account> accounts{acct1, acct2, acct3};
    std::vector<Transaction> transactions{trans1, trans2, trans3};

    return Ledger{accounts, transactions};
}

TEST(Find_account_finds_account)
{
    Ledger ledger = make_test_ledger();

    Account& acct1 = find_account(ledger, 1);
    CHECK_EQUAL("Jesse", acct1.owner);

    Account& acct2 = find_account(ledger, 2);
    CHECK_EQUAL("Shu-Hung", acct2.owner);

    Account& acct3 = find_account(ledger, 4);
    CHECK_EQUAL("Majed", acct3.owner);
}

TEST(Find_account_throws_for_bad_acct_id)
{
    Ledger ledger = make_test_ledger();

    CHECK_THROW(find_account(ledger, 0), Bank_error);
    CHECK_THROW(find_account(ledger, 3), Bank_error);
    CHECK_THROW(find_account(ledger, 5), Bank_error);
}

TEST(Execute_transaction_credit)
{
    Account acct{2, 100, "Jesse"};
    Transaction trans{2, 10, Transaction_type::credit, 5};

    execute_transaction(acct, trans);

    CHECK_EQUAL(2, acct.id);
    CHECK_EQUAL(110, acct.balance);
    CHECK_EQUAL("Jesse", acct.owner);
}

TEST(Execute_transaction_debit)
{
    Account acct{2, 100, "Jesse"};
    Transaction trans{2, 10, Transaction_type::debit, 5};

    execute_transaction(acct, trans);

    CHECK_EQUAL(2, acct.id);
    CHECK_EQUAL(90, acct.balance);
    CHECK_EQUAL("Jesse", acct.owner);
}

TEST(Execute_transaction_mismatch_throws)
{
    Account acct{2, 100, "Jesse"};
    Transaction trans{3, 10, Transaction_type::debit, 5};

    CHECK_THROW(execute_transaction(acct, trans), Bank_error);
}

TEST(Clear_transactions)
{
    Ledger ledger = make_test_ledger();
    clear_transactions(ledger);

    CHECK_EQUAL(0, ledger.transactions.size());

    CHECK_EQUAL(140,    ledger.accounts[0].balance);
    CHECK_EQUAL(10'000, ledger.accounts[1].balance);
    CHECK_EQUAL(12'000, ledger.accounts[2].balance);
}
